#import <Cocoa/Cocoa.h>
#import "BWFileList.h"
#import "BWLog.h"

static const char *cvsid = "$Id: BWFileList.m,v 1.1.1.1 2002/03/25 01:36:49 markd Exp $";
static BWLogModule *entryLogger;
static BWLogModule *logger;


@implementation BWFileEntry

+ (void) initialize
{
    entryLogger = [BWLogModule registerModule: @"BWFileEntry" 
			       withCVSId: cvsid];
} // initalize


+ (BWFileEntry *) newEntryWithPath: (NSString *) path  basedAt: (NSString *) at
{
    NSFileManager *manager = [NSFileManager defaultManager];
    BWFileEntry *entry = nil;

    BWLog (entryLogger, BWTrace, @"newEntryWithPath: %@ at %@", path, at);

    if ([manager fileExistsAtPath: path]) {
	NSDictionary *attributes = [manager fileAttributesAtPath: path
					    traverseLink: YES];
	int size = [[attributes objectForKey: NSFileSize] intValue];

	entry = [[BWFileEntry alloc] initWithRoot: at
				     relativePath: path
				     fileName: [path lastPathComponent]
				     fileSize: size];
    }

    return ([entry autorelease]);
    
} // newEntryWithPath


- (id) initWithRoot: (NSString *) r  relativePath: (NSString *) rpath
	   fileName: (NSString *) name  fileSize: (int) size
{
    BWLog (entryLogger, BWTrace, @"initWithRoot: %@, relativePath: %@, name: %@, fiileSize: %d", r, rpath, name, size);

    if (self = [super init]) {
	root = [r retain];
	relativePath = [rpath retain];
	fileName = [name retain];
	fileSize = size;
    }

    return (self);

} // initWithEverything, hold the pickles


- (void) dealloc
{
    BWLog (entryLogger, BWTrace, @"dealloc");

    [root release];
    [relativePath release];
    [fileName release];

} // dealloc


- (NSString *) description
{
    NSString *desc;
    desc = [NSString stringWithFormat: @"name: %@  size: %d  relativePath: %@",
		     fileName, fileSize, relativePath];
    return (desc);

} // description


- (NSString *) fullPath
{
    return ([root stringByAppendingFormat: @"/%@", relativePath]);

} // fullPath



- (NSString *) fileName
{
    return (fileName);

} // fileName


- (int) fileSize
{
    return (fileSize);

} // fileSize


- (NSComparisonResult) compareByName: (BWFileEntry *) thing2
{
    return ([[self fileName] caseInsensitiveCompare: [thing2 fileName]]);
} // compareByName


@end // BWFileEntry




@implementation BWFileList

+ (void) initialize
{
    logger = [BWLogModule registerModule: @"BWFileList" 
			  withCVSId: cvsid];
} // initalize


+ (BWFileList *) loadFromDirectory: (NSString *) pathName  deeply: (BOOL) deeply
{
    NSDirectoryEnumerator *dirent;
    NSString *file;
    BWFileList *list;
    NSFileManager *manager = [NSFileManager defaultManager];
    NSMutableArray *files;
    NSMutableArray *imageFileTypes;

    BWLog (logger, BWTrace, @"loadFromDirectroy: %@  deeply: %d", pathName, deeply);

    files = [NSMutableArray arrayWithCapacity: 5000];

    dirent = [manager enumeratorAtPath: pathName];

    if (!deeply) {
	[dirent skipDescendents];
    }

    // I don't want pdfs.  deal with it.
    imageFileTypes = [[NSMutableArray alloc] init];
    [imageFileTypes addObjectsFromArray: [NSImage imageFileTypes]];
    [imageFileTypes removeObject: @"pdf"];
    [imageFileTypes removeObject: @"PDF"];

    BWLog (logger, BWVerbose, @"the file types looked for: %@", imageFileTypes);

    [manager changeCurrentDirectoryPath: pathName];

    while (file = [dirent nextObject]) {
	NSString *extension = [file pathExtension];

	if ([imageFileTypes indexOfObject: extension] != NSNotFound) {
	    BWFileEntry *entry;
	    entry = [BWFileEntry newEntryWithPath: file  basedAt: pathName];
	    [files addObject: entry];
	}
    }
    [imageFileTypes release];

    BWLog (logger, BWTrace, @"file list item count: %d", [files count]);

    list = [[BWFileList alloc] initWithBasePath: pathName  relativeFileList: files];

    return (list);

} // loadFromDirectory



- (id) initWithBasePath: (NSString *) base relativeFileList: (NSArray *) list
{
    BWLog (logger, BWTrace, @"initWithBasePath: %@", base);
    BWLog (logger, BWVerbose, @"and with relative file list: %@", list);

    if (self = [super init]) {
	basePath = [base retain];
	fileList = [list retain];
    }

    return (self);

} // initWithBasePath


- (int) count
{
    return ([fileList count]);

} // count



- (BWFileEntry *) getFileAt: (int) index
{
    return ([fileList objectAtIndex: index]);

} // getFileAt


- (void) removeFileAt: (int) index
{
    [fileList removeObjectAtIndex: index];;
} // removeFileAt



- (void) sortByFileName
{
    BWLog (logger, BWTrace, @"sorting file list by entry name");

    [fileList sortUsingSelector: @selector(compareByName:)];

} // sortByFileName



@end // BWFileList


